#!/usr/bin/python

import sys
sys.path.append("/usr/share/rhn/")
from up2date_client import up2date

# mark this module as acceptable
__rhnexport__ = [
    'update',
    'remove',
    'refresh_list'
]

argVerbose = 0  # verbose output (might clutter syslog)

def remove(pkgList):
    """We have been told that we should remove packages"""
    def rpmCallback(what, amount, total, hdr, path):
        if what == rpm.RPMCALLBACK_INST_OPEN_FILE:
            global fd
            fileName = "%s/%s-%s-%s.%s.rpm" % (path,
                                               hdr['name'],
                                               hdr['version'],
                                               hdr['release'],
                                               hdr['arch'])
            try:
                 fd = os.open(fileName, os.O_RDONLY)
            except:
                print "Error opening %s for reading" % fileName
                return -1

            return fd
        elif what == rpm.RPMCALLBACK_INST_START:
            fileName = "%s-%s-%s.%s.rpm" % (hdr['name'],
                                            hdr['version'],
                                            hdr['release'],
                                            hdr['arch'])
        elif what == rpm.RPMCALLBACK_INST_CLOSE_FILE:
            os.close(fd)

        elif what == rpm.RPMCALLBACK_INST_PROGRESS:
            pass
        elif what == rpm.RPMCALLBACK_UNINST_STOP:
            pass

    if type(pkgList) != type([]):
        return (13, "Invalid arguments passed to function",{})

    if argVerbose > 1:
        print "Called remove_packages(%s)" % (pkgList,)

    # FIXME: figure out the error handling
    try:
        up2date.removePackages(pkgList, rpmCallback)
    except up2date.DependencyError, e:
        # just return the list of deps if we failed
        return (14, "%s" % e, {})
    except up2date.RpmRemoveError, e:
        # if we later need finer grained info, we can
        return (15, "%s" % e, {})
    except up2date.RpmRemoveSkipListError, e:
        return (16, "%s could not be removed because it was on the skip list" % e, {})

    # if were succesful, return "4"
    return (0, "%s installed successfully" % pkgList, {})

def update(pkgList):        
    """We have been told that we should retrieve/install packages"""

    def rpmCallback(what, amount, total, hdr, path):
        if what == rpm.RPMCALLBACK_INST_OPEN_FILE:
            global fd
            fileName = "%s/%s-%s-%s.%s.rpm" % (path,
                                               hdr['name'],
                                               hdr['version'],
                                               hdr['release'],
                                               hdr['arch'])
            try:
                 fd = os.open(fileName, os.O_RDONLY)
            except:
                print "Error opening %s for reading" % fileName
                return -1

            return fd
        elif what == rpm.RPMCALLBACK_INST_START:
            fileName = "%s-%s-%s.%s.rpm" % (hdr['name'],
                                            hdr['version'],
                                            hdr['release'],
                                            hdr['arch'])
        elif what == rpm.RPMCALLBACK_INST_CLOSE_FILE:
            os.close(fd)

        elif what == rpm.RPMCALLBACK_INST_PROGRESS:
            pass
        elif what == rpm.RPMCALLBACK_UNINST_STOP:
            pass

    if type(pkgList) != type([]):
        return (13, "Invalid arguments passed to function", {})

    if argVerbose > 1:
        print "Called update_packages(%s)" % (pkgList,)

    # if e,v,r is "", pretend it is version 0, release 0, epoch ""
    for pkg in pkgList:
        if pkg[1] == pkg[2] == pkg[3] == "":
            pkg[1] = pkg[2] = 0
            pkg[3] = ""

    try:
        ret = batchRun(0, pkgList)
    # catch all the exceptions we raise, and hopefully return a useful error
    # message to the server
    except up2date.RpmError, e:
        return (18, "Failed: packages requested raised  dependency problems", {})
    except up2date.CommunicationError, e:
        return (21, "Failed: There was a communication error talking to the server", {})
    except up2date.GPGVerificationUnsignedPackageError, e:
        return (22, "Failed: There was a package gpg verification error. The error was: %s" % e, {})
    except up2date.GPGVerificationError, e:
        return (23, "Failed: There was a package gpg verification error. The error was: %s" % e, {})
    except up2date.GPGInstallationError:
        return (24, "Failed: gpg is not properly installed on this machine", {})
    except up2date.GPGKeyringError:
        return (25, "Failed: The package signing key for Red Hat, Inc. is not on the gpg keyring", {})
 
    return (0, "Packages were installed successfully", {})


def batchRun(onlyList, pkgList):
    from up2date_client import wrapper
    ret = wrapper.batchRun(onlyList, [], 0, fromDaemon = 1, actionPkgs = pkgList)
    return ret

# push again the list of rpm packages to the server
def refresh_list():
    s = up2date.getServer()

    if argVerbose > 1:
        print "Called refresh_rpmlist"

    ret = None
    try:
        s.registration.update_packages(up2date.getSystemId(),
                                       up2date.getInstalledPackageList())
    except:
        print "ERROR: refreshing remote package list for System Profile"
        return (20, "Error refreshing package list", {})

    return (0, "rpmlist refreshed", {})
